# Supports Organizer.rb
=begin
Copyright 2009, Matt
All Rights Reserved

THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

License:   Free

Author:     	Matthieu NOBLET

Organization:   Matt666

Name:      		Make_components

Version:   		1.1

SU Version:   	6 and 7

Date:      		12/03/2009

Description:   	Converts all entities into components in the current selection

Usage:      	Converts one entity into one group
				Converts one entity into one component
				If one group is selected, use native converter
				Converts all entities in the selection into single components
				Converts all entities in the selection into single groups
				
History:
	1.000   11/03/2009
			First version
			
	1.100	12/03/2009
			Bug fixed (thank you Pilou!)
			Message box added to choose if entities in the selection must be converted into components
			Make a group of just one entity
			Make a component of just one entity
			Convert all entities into single groups
=end

require 'sketchup.rb'

unless file_loaded?(__FILE__)
	file_loaded(__FILE__)
	$submenu ? (organizer = $submenu) : (organizer = UI.menu("Plugins"))
	organizer.add_separator
	organizer.add_item("Convert in single components"){ Matt_Tools.make_components "Components" }
	organizer.add_item("Convert in single groups"){ Matt_Tools.make_components "Groups" }
	organizer.add_item("Make component"){ Matt_Tools.make_component "Components"}
	organizer.add_item("Make group"){ Matt_Tools.make_component "Groups"}
	
	UI.add_context_menu_handler do |menu|
		menu.add_separator if (Matt_Tools.make_components_valid) or (Matt_Tools.make_component_valid "Components") or (Matt_Tools.make_component_valid "Groups") or (Matt_Tools.make_components_valid)
		menu.add_item("Convert in single components"){ Matt_Tools.make_components "Components" } if (Matt_Tools.make_components_valid)
		menu.add_item("Convert in single groups"){ Matt_Tools.make_components "Groups" } if (Matt_Tools.make_components_valid)
		menu.add_item ("Make component"){ Matt_Tools.make_component "Components"} if (Matt_Tools.make_component_valid "Components")
		menu.add_item ("Make group"){ Matt_Tools.make_component "Groups"} if (Matt_Tools.make_component_valid "Groups")
	end
end

module Matt_Tools
	def self.make_component(name)
		model = Sketchup.active_model
		if model.selection.empty?
			return nil
		else
			e = model.selection[0]
			if Sketchup.version[0,1].to_i >= 7
				model.start_operation(("Make " + name),true)
			else
				model.start_operation("Make " + name)
			end
			if e.respond_to?("curve") and e.curve
				g = model.entities.add_group e.curve.edges
			else
				g = model.entities.add_group e
			end
			if name=="Groups" && e.class==Sketchup::ComponentInstance
				g.name = e.definition.name 
				e.explode 
			elsif name=="Components"
				g = g.to_component
				puts g
			end
			model.selection.clear
			model.selection.add(g)
			model.commit_operation
		end
	end
	
	def self.make_components(name)
		model = Sketchup.active_model
		sel = model.selection
		if sel.empty?
			return nil
		else
			if Sketchup.version[0,1].to_i >= 7
				model.start_operation(("Make " + name),true)
			else
				model.start_operation("Make " + name)
			end
			n=0
			n2=0
			l = []
			if self.make_components_entities(name) && UI.messagebox(("Convert all entities in the selection into "+name+"?"),MB_YESNO,("Convert ents into "+name+"..."))==6
				sel.each do |e|
					g = nil
					if not(l.index e)
						if e.class==Sketchup::Group && name=="Components"
							e.to_component
							n+=1
						elsif e.class==Sketchup::ComponentInstance && name=="Groups"
							g = model.entities.add_group e
							g.name = e.definition.name
							e.explode
							n+=1
						elsif e.class==Sketchup::Edge
							if e.curve
								l.push(e.curve.edges).flatten!
								g = model.active_entities.add_group e.curve.edges
							else
								g = model.active_entities.add_group e
							end
							g.to_component if name=="Components"
							n2+=1
						elsif e.is_a?(Sketchup::Drawingelement) and not [Sketchup::ComponentInstance,Sketchup::Group].index(e.class)
							g = model.active_entities.add_group e
							g.to_component if name=="Components"
							n2+=1
						end
					end
				end
			else
				sel.each do |e|
					g = nil
					if e.class==Sketchup::Group && name=="Components"
						nm = e.name
						e.to_component
						e.definition.name = nm if nm
						n+=1
					elsif e.class==Sketchup::ComponentInstance && name=="Groups"
						g = model.entities.add_group e
						g.name=e.definition.name
						e.explode
						n+=1
					end
				end
			end
			if name=="Groups";name2 = "Components";else;name2 = "Groups";end
			UI.messagebox(n.to_s + " " + name2 + " converted into " + name + ".\n" + n2.to_s + " entities converted into " + name + ".")
			model.commit_operation
		end
	end
	
	def self.make_components_valid
		sel = Sketchup.active_model.selection
		return false if sel.length==1
		return true
	end
	
	def self.make_component_valid(name)
		sel = Sketchup.active_model.selection
		e = sel[0]
		n = e.curve.edges.length if(e.respond_to?("curve") and e.curve)
		if sel.length==1 or(n and n==sel.length)
			if name=="Components"
				return true if not([Sketchup::Group,Sketchup::ComponentInstance].index(sel[0].class))
			else
				return true if not(sel[0].class==Sketchup::Group)
			end
		else
			return false
		end
		return false
	end
	
	def self.make_components_entities(name)
		Sketchup.active_model.selection.each{|e|return true if e.class!=Sketchup::Group} if name=="Components"
		Sketchup.active_model.selection.each{|e|return true if e.class!=Sketchup::ComponentInstance} if name=="Groups"
		return false
	end
end
